# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: skip-file
# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class RepositoryMetadataProperties(Model):
    """RepositoryMetadataProperties.

    :param id:
    :type id: str
    :param tenant_id:
    :type tenant_id: str
    :param tenant_name:
    :type tenant_name: str
    :param kind: Possible values include: 'Repository', 'User', 'Key'
    :type kind: str or ~digitaltwinrepositoryprovisioningservice.models.enum
    :param created_on:
    :type created_on: datetime
    :param last_updated:
    :type last_updated: datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'tenant_id': {'key': 'tenantId', 'type': 'str'},
        'tenant_name': {'key': 'tenantName', 'type': 'str'},
        'kind': {'key': 'kind', 'type': 'str'},
        'created_on': {'key': 'createdOn', 'type': 'iso-8601'},
        'last_updated': {'key': 'lastUpdated', 'type': 'iso-8601'},
    }

    def __init__(self, id=None, tenant_id=None, tenant_name=None, kind=None, created_on=None, last_updated=None):
        super(RepositoryMetadataProperties, self).__init__()
        self.id = id
        self.tenant_id = tenant_id
        self.tenant_name = tenant_name
        self.kind = kind
        self.created_on = created_on
        self.last_updated = last_updated
