# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: skip-file
# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class RepositoryUpsertRequest(Model):
    """Repository Provision Upsert request object.

    :param properties:
    :type properties:
     ~digitaltwinrepositoryprovisioningservice.models.RepositoryUpsertRequestProperties
    """

    _attribute_map = {
        'properties': {'key': 'properties', 'type': 'RepositoryUpsertRequestProperties'},
    }

    def __init__(self, properties=None):
        super(RepositoryUpsertRequest, self).__init__()
        self.properties = properties
